@@ -31,10 +31,11 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.80';
+$VERSION = '1.95';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
+sub ProcessMRAW($$$);
 
 # the following RAF version numbers have been tested for writing:
 # (as of ExifTool 11.70, this lookup is no longer used if the version number is numerical)
@@ -42,11 +43,12 @@ my %testedRAF = (
     '0100' => 'E550, E900, F770, S5600, S6000fd, S6500fd, HS10/HS11, HS30, S200EXR, X100, XF1, X-Pro1, X-S1, XQ2 Ver1.00, X-T100, GFX 50R, XF10',
     '0101' => 'X-E1, X20 Ver1.01, X-T3',
     '0102' => 'S100FS, X10 Ver1.02',
-    '0103' => 'IS Pro Ver1.03',
+    '0103' => 'IS Pro and X-T5 Ver1.03',
     '0104' => 'S5Pro Ver1.04',
     '0106' => 'S5Pro Ver1.06',
     '0111' => 'S5Pro Ver1.11',
     '0114' => 'S9600 Ver1.00',
+    '0120' => 'X-T4 Ver1.20',
     '0132' => 'X-T2 Ver1.32',
     '0144' => 'X100T Ver1.44',
     '0159' => 'S2Pro Ver1.00',
@@ -222,6 +224,7 @@ my %faceCategories = (
     },
     0x100a => { #2
         Name => 'WhiteBalanceFineTune',
+        Notes => 'newer cameras should divide these values by 20', #forum10800
         Writable => 'int32s',
         Count => 2,
         PrintConv => 'sprintf("Red %+d, Blue %+d", split(" ", $val))',
@@ -254,6 +257,23 @@ my %faceCategories = (
             0x2e0 => '-4 (weakest)', #10 (-4)
         },
     },
+    0x100f => { #PR158
+        Name => 'Clarity',
+        Writable => 'int32s', #PH
+        PrintConv => {
+            -5000 => '-5',
+            -4000 => '-4',
+            -3000 => '-3',
+            -2000 => '-2',
+            -1000 => '-1',
+            0 => '0',
+            1000 => '1',
+            2000 => '2',
+            3000 => '3',
+            4000 => '4',
+            5000 => '5',
+        },
+    },
     0x1010 => {
         Name => 'FujiFlashMode',
         Writable => 'int16u',
@@ -410,6 +430,14 @@ my %faceCategories = (
         Name => 'ShadowTone',
         Writable => 'int32s',
         PrintConv => {
+            OTHER => sub {
+                my ($val, $inv) = @_;
+                if ($inv) {
+                    return int(-$val * 16);
+                } else {
+                    return -$val / 16;
+                }
+            },
             -64 => '+4 (hardest)',
             -48 => '+3 (very hard)',
             -32 => '+2 (hard)',
@@ -423,6 +451,14 @@ my %faceCategories = (
         Name => 'HighlightTone',
         Writable => 'int32s',
         PrintConv => {
+            OTHER => sub {
+                my ($val, $inv) = @_;
+                if ($inv) {
+                    return int(-$val * 16);
+                } else {
+                    return -$val / 16;
+                }
+            },
             -64 => '+4 (hardest)',
             -48 => '+3 (very hard)',
             -32 => '+2 (hard)',
@@ -444,7 +480,7 @@ my %faceCategories = (
         PrintConv => { 0 => 'Off', 1 => 'On' },
     },
     0x1047 => { #12
-        Name => 'GrainEffect',
+        Name => 'GrainEffectRoughness',
         Writable => 'int32s',
         PrintConv => {
             0 => 'Off',
@@ -461,14 +497,29 @@ my %faceCategories = (
             64 => 'Strong',
         },
     },
-    0x1049 => { #12
+    0x1049 => { #12,forum14319
         Name => 'BWAdjustment',
         Notes => 'positive values are warm, negative values are cool',
         Format => 'int8s',
         PrintConv => '$val > 0 ? "+$val" : $val',
         PrintConvInv => '$val + 0',
     },
-    # 0x104b - BWAdjustment for Green->Magenta (forum10800)
+    0x104b => { #forum10800,forum14319
+        Name => 'BWMagentaGreen',
+        Notes => 'positive values are green, negative values are magenta',
+        Format => 'int8s',
+        PrintConv => '$val > 0 ? "+$val" : $val',
+        PrintConvInv => '$val + 0',
+    },
+    0x104c => { #PR158
+        Name => "GrainEffectSize",
+        Writable => 'int16u', #PH
+        PrintConv => {
+            0 => 'Off',
+            16 => 'Small',
+            32 => 'Large',
+        },
+    },
     0x104d => { #forum9634
         Name => 'CropMode',
         Writable => 'int16u',
@@ -477,6 +528,16 @@ my %faceCategories = (
             1 => 'Full-frame on GFX', #IB
             2 => 'Sports Finder Mode', # (mechanical shutter)
             4 => 'Electronic Shutter 1.25x Crop', # (continuous high)
+            8 => 'Digital Tele-Conv', #forum15784
+        },
+    },
+    0x104e => { #forum10800 (X-Pro3)
+        Name => 'ColorChromeFXBlue',
+        Writable => 'int32s',
+        PrintConv => {
+            0 => 'Off',
+            32 => 'Weak', # (NC)
+            64 => 'Strong',
         },
     },
     0x1050 => { #forum6109
@@ -489,6 +550,16 @@ my %faceCategories = (
             3 => 'Electronic Front Curtain', #10
         },
     },
+    0x1051 => { #forum15784
+        Name => 'CropFlag',
+        Writable => 'int8u',
+        Notes => q(
+            this tag exists only if the image was cropped, and is 0 for cropped JPG
+            image or 1 for a cropped RAF
+        ),
+    },
+    0x1052 => { Name => 'CropTopLeft', Writable => 'int32u' }, #forum15784
+    0x1053 => { Name => 'CropSize',    Writable => 'int32u' }, #forum15784
     # 0x1100 - This may not work well for newer cameras (ref forum12682)
     0x1100 => [{
         Name => 'AutoBracketing',
@@ -628,6 +699,7 @@ my %faceCategories = (
             0x800 => 'Classic Negative', #forum10536
             0x900 => 'Bleach Bypass', #forum10890
             0xa00 => 'Nostalgic Neg', #forum12085
+            0xb00 => 'Reala ACE', #forum15190
         },
     },
     0x1402 => { #2
@@ -673,15 +745,6 @@ my %faceCategories = (
         PrintConv => '"$val%"',
         PrintConvInv => '$val=~s/\s*\%$//; $val',
     },
-    0x104e => { #forum10800 (X-Pro3)
-        Name => 'ColorChromeFXBlue',
-        Writable => 'int32s',
-        PrintConv => {
-            0 => 'Off',
-            32 => 'Weak', # (NC)
-            64 => 'Strong',
-        },
-    },
     0x1422 => { #8
         Name => 'ImageStabilization',
         Writable => 'int16u',
@@ -689,8 +752,9 @@ my %faceCategories = (
         PrintConv => [{
             0 => 'None',
             1 => 'Optical', #PH
-            2 => 'Sensor-shift', #PH
+            2 => 'Sensor-shift', #PH (now IBIS/OIS, ref forum13708)
             3 => 'OIS Lens', #forum9815 (optical+sensor?)
+            258 => 'IBIS/OIS + DIS', #forum13708 (digital on top of IBIS/OIS)
             512 => 'Digital', #PH
         },{
             0 => 'Off',
@@ -762,6 +826,9 @@ my %faceCategories = (
         },
         PrintConvInv => '$val=~/(0x[0-9a-f]+)/i; hex $1',
     },
+    0x1447 => { Name => 'FujiModel',  Writable => 'string' },
+    0x1448 => { Name => 'FujiModel2', Writable => 'string' },
+    0x144d => { Name => 'RollAngle',  Writable => 'rational64s' }, #forum14319
     0x3803 => { #forum10037
         Name => 'VideoRecordingMode',
         Groups => { 2 => 'Video' },
@@ -771,6 +838,7 @@ my %faceCategories = (
             0x00 => 'Normal',
             0x10 => 'F-log',
             0x20 => 'HLG',
+            0x30 => 'F-log2', #forum14384
         },
     },
     0x3804 => { #forum10037
@@ -841,6 +909,26 @@ my %faceCategories = (
             1 => 'Face',
             2 => 'Left Eye',
             3 => 'Right Eye',
+            7 => 'Body',
+            8 => 'Head',
+            11 => 'Bike',
+            12 => 'Body of Car',
+            13 => 'Front of Car',
+            14 => 'Animal Body',
+            15 => 'Animal Head',
+            16 => 'Animal Face',
+            17 => 'Animal Left Eye',
+            18 => 'Animal Right Eye',
+            19 => 'Bird Body',
+            20 => 'Bird Head',
+            21 => 'Bird Left Eye',
+            22 => 'Bird Right Eye',
+            23 => 'Aircraft Body',
+            25 => 'Aircraft Cockpit',
+            26 => 'Train Front',
+            27 => 'Train Cockpit',
+            28 => 'Animal Head (28)', #forum15192
+            29 => 'Animal Body (29)', #forum15192
         },'REPEAT'],
     },
     # 0x4202 int8u[-1] - number of cooredinates in each rectangle? (ref 11)
@@ -953,14 +1041,19 @@ my %faceCategories = (
     },
     0.5 => {
         Name => 'AFAreaZoneSize',
-        Mask => 0xf0000,
+        Mask => 0xff0000,
         PrintConv => {
             0 => 'n/a',
             OTHER => sub {
                 my ($val, $inv) = @_;
-                return "$val x $val" unless $inv;
-                $val =~ s/ ?x.*//;
-                return $val;
+                my ($w, $h);
+                if ($inv) {
+                    my ($w, $h) = $val =~ /(\d+)/g;
+                    return 0 unless $w and $h;
+                    return((($h << 5) & 0xf0) | ($w & 0x0f));
+                }
+                ($w, $h) = ($val & 0x0f, $val >> 5);
+                return "$w x $h";
             },
         },
     },
@@ -1025,7 +1118,7 @@ my %faceCategories = (
         Mask => 0x000000ff,
         PrintConv => {
             0 => 'Single',
-            1 => 'Continuous Low',
+            1 => 'Continuous Low', # not used by X-H2S? (see forum13777)
             2 => 'Continuous High',
         },
     },
@@ -1076,6 +1169,46 @@ my %faceCategories = (
     Face8Birthday => { },
 );
 
+# tags extracted from RAF header
+%Image::ExifTool::FujiFilm::RAFHeader = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'RAF', 1 => 'RAF', 2 => 'Image' },
+    NOTES => 'Tags extracted from the header of RAF images.',
+  # 0x00 - eg. "FUJIFILMCCD-RAW 0201FA392001FinePix S3Pro"
+    0x3c => { #PH
+        Name => 'RAFVersion',
+        Format => 'undef[4]',
+    },
+    # (all int32u values)
+  # 0x40 - 1 for M-RAW, 0 otherwise?
+  # 0x44 - high word of M-RAW offset? (only seen zero)
+  # 0x48 - M-RAW header offset
+  # 0x4c - M-RAW header length
+  # 0x50 - ? (only seen zero)
+  # 0x54 - JPEG offset
+  # 0x58 - JPEG length
+  # 0x5c - RAF directory offset
+  # 0x60 - RAF directory length
+  # 0x64 - FujiIFD dir offset
+  # 0x68 - FujiIFD dir length
+  # 0x6c - RAFCompression or JPEG start
+    0x6c => { #10
+        Name => 'RAFCompression',
+        Condition => '$$valPt =~ /^\0\0\0/', # (JPEG header is in this location for some RAF versions)
+        Format => 'int32u',
+        PrintConv => { 0 => 'Uncompressed', 2 => 'Lossless', 3 => 'Lossy'  },
+    },
+  # 0x70 - ? same as 0x68?
+  # 0x74 - ? usually 0, but have seen 0x1700
+  # 0x78 - RAF1 dir offset
+  # 0x7c - RAF1 dir length
+  # 0x80 - FujiIFD1 dir offset
+  # 0x84 - FujiIFD1 dir length
+  # 0x88-0x8c - always zero?
+  # 0x90 - ? same as 0x74?
+  # 0x94 - JPEG or M-RAW start
+);
+
 # tags in RAF images (ref 5)
 %Image::ExifTool::FujiFilm::RAF = (
     PROCESS_PROC => \&ProcessFujiDir,
@@ -1110,6 +1243,8 @@ my %faceCategories = (
         ValueConv => 'my @v=reverse split(" ",$val);"@v"', # reverse to show width first
         PrintConv => '$val=~tr/ /x/; $val',
     },
+    # 0x112 - int16u[2] same as 0x111 but with width/height swapped?
+    # 0x113 - int16u[2] same as 0x111?
     0x115 => {
         Name => 'RawImageAspectRatio',
         Format => 'int16u',
@@ -1159,6 +1294,7 @@ my %faceCategories = (
         Count => 36,
         PrintConv => '$val =~ tr/012 /RGB/d; join " ", $val =~ /....../g',
     },
+    # 0x141 - int16u[2] Bit depth? "14 42" for 14-bit RAF and "16 48" for 16-bit RAF
     0x2000 => { #IB
         Name => 'WB_GRGBLevelsAuto',
         Format => 'int16u',
@@ -1316,6 +1452,7 @@ my %faceCategories = (
     0xf007 => {
         Name => 'StripOffsets',
         IsOffset => 1,
+        IsImageData => 1,
         OffsetPair => 0xf008,  # point to associated byte counts
     },
     0xf008 => {
@@ -1330,6 +1467,8 @@ my %faceCategories = (
     0xf00e => 'WB_GRBLevels',
     0xf00f => 'ChromaticAberrationParams', # (rational64s[23])
     0xf010 => 'VignettingParams', #9 (rational64s[31 or 64])
+    # 0xf013 - int32u[3] same as 0xf00d
+    # 0xf014 - int32u[3] - also related to WhiteBalance
 );
 
 # information found in FFMV atom of MOV videos
@@ -1378,6 +1517,30 @@ my %faceCategories = (
     },
 );
 
+# tags in RAF M-RAW header (ref PH)
+%Image::ExifTool::FujiFilm::MRAW = (
+    PROCESS_PROC => \&ProcessMRAW,
+    GROUPS => { 0 => 'RAF', 1 => 'M-RAW', 2 => 'Image' },
+    FORMAT => 'int32u',
+    TAG_PREFIX => 'MRAW',
+    NOTES => q{
+        Tags extracted from the M-RAW header of multi-image RAF files.  The family 1
+        group name for these tags is "M-RAW".
+    },
+    1 => { Name => 'RawImageNumber', Format => 'int32u' },
+    # 3 - seen "0 100", "-300 100" and "300 100" for a sequence of 3 images
+    3 => { Name => 'ExposureCompensation', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
+    # 4 - (same value as 3 in all my samples)
+    4 => { Name => 'ExposureCompensation2', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
+    # 5 - seen "10 1600", "10 6800", "10 200", "10 35000" etc
+    5 => { Name => 'ExposureTime', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)' },
+    # 6 - seen "450 100", "400 100" (all images in RAF have same value)
+    6 => { Name => 'FNumber', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintFNumber($val)' },
+    # 7 - seen 200, 125, 250, 2000
+    7 => 'ISO',
+    # 8 - seen 0
+);
+
 #------------------------------------------------------------------------------
 # decode information from FujiFilm face recognition information
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference, 2) tag table ref
@@ -1449,7 +1612,7 @@ sub ProcessFujiDir($$$)
     $raf->Read($buff, 4) or return 0;
     my $entries = unpack 'N', $buff;
     $entries < 256 or return 0;
-    $et->Options('Verbose') and $et->VerboseDir('Fuji', $entries);
+    $et->VerboseDir('Fuji', $entries);
     SetByteOrder('MM');
     my $pos = $offset + 4;
     for ($index=0; $index<$entries; ++$index) {
@@ -1481,6 +1644,51 @@ sub ProcessFujiDir($$$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# get information from FujiFilm M-RAW header
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 if this was a valid M-RAW headerx
+sub ProcessMRAW($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dataPos = $$dirInfo{DataPos};
+    my $dataLen = length $$dataPt;
+    $dataLen < 44 and $et->Warn('Short M-RAW header'), return 0;
+    $$dataPt =~ /^FUJIFILMM-RAW  / or $et->Warn('Bad M-RAW header'), return 0;
+    my $ver = substr($$dataPt, 16, 4);
+    $et->VerboseDir("M-RAW $ver", undef, $dataLen);
+    SetByteOrder('MM');
+    my $size = Get16u($dataPt, 40); # (these are just a guess - PH)
+    my $num = Get16u($dataPt, 42);
+    my $pos = 44;
+    my ($i, $n);
+    for ($n=0; ; ++$n) {
+        $pos += 16;  # skip offset/size fields
+        my $end = $pos + $size;
+        last if $end > $dataLen;
+        $$et{DOC_NUM} = ++$$et{DOC_COUNT} if $pos > 60;
+        $et->VPrint(0, "$$et{INDENT}(Raw image $n parameters: $size bytes, $num entries)\n");
+        for ($i=0; $i<$num; ++$i) {
+            last if $pos + 4 > $end;
+            # (don't know what the byte at the current $pos is for, value = 0x20)
+            my $tag = Get8u($dataPt, $pos+1);
+            my $size = Get16u($dataPt, $pos+2);
+            $pos += 4;
+            last if $pos + $size > $end;
+            $et->HandleTag($tagTablePtr, $tag, undef,
+                DataPt  => $dataPt,
+                DataPos => $dataPos,
+                Start   => $pos,
+                Size    => $size,
+            );
+            $pos += $size;
+        }
+    }
+    delete $$et{DOC_NUM};
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # write information to FujiFilm RAW file (RAF)
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -1496,10 +1704,12 @@ sub WriteRAF($$)
     my $ver = substr($hdr, 0x3c, 4);
     $ver =~ /^\d{4}$/ or $testedRAF{$ver} or return 0;
 
+    # get position and size of M-RAW header
+    my ($mpos, $mlen) = unpack('x72NN', $hdr);
     # get the position and size of embedded JPEG
     my ($jpos, $jlen) = unpack('x84NN', $hdr);
     # check to be sure the JPEG starts in the expected location
-    if ($jpos > 0x94 or $jpos < 0x68 or $jpos & 0x03) {
+    if (($mpos > 0x94 or $jpos > 0x94 + $mlen) or $jpos < 0x68 or $jpos & 0x03) {
         $et->Error("Unsupported or corrupted RAF image (version $ver)");
         return 1;
     }
@@ -1513,12 +1723,33 @@ sub WriteRAF($$)
         $et->Error('Error reading RAF meta information');
         return 1;
     }
+    if ($mpos) {
+        if ($mlen != 0x11c) {
+            $et->Error('Unsupported M-RAW header (please submit sample for testing)');
+            return 1;
+        }
+        # read M-RAW header and add to file header
+        my $mraw;
+        unless ($raf->Seek($mpos, 0) and $raf->Read($mraw, $mlen) == $mlen) {
+            $et->Error('Error reading M-RAW header');
+            return 1;
+        }
+        $hdr .= $mraw;
+        # verify that the 1st raw image offset is zero, and that the 1st raw image
+        # length is the same as the 2nd raw image offset
+        unless (substr($hdr, 0xc0, 8) eq "\0\0\0\0\0\0\0\0" and
+                substr($hdr, 0xc8, 8) eq substr($hdr, 0x110, 8))
+        {
+            $et->Error('Unexpected layout of M-RAW header');
+            return 1;
+        }
+    }
     # use same write directories as JPEG
     $et->InitWriteDirs('JPEG');
     # rewrite the embedded JPEG in memory
     my %jpegInfo = (
         Parent  => 'RAF',
-        RAF     => new File::RandomAccess(\$jpeg),
+        RAF     => File::RandomAccess->new(\$jpeg),
         OutFile => \$outJpeg,
     );
     $$et{FILE_TYPE} = 'JPEG';
@@ -1557,12 +1788,28 @@ sub WriteRAF($$)
     }
     # calculate offset difference due to change in JPEG size
     my $ptrDiff = length($outJpeg) + length($pad) - ($jlen + $oldPadLen);
-    # update necessary pointers in header
-    foreach $offset (0x5c, 0x64, 0x78, 0x80) {
+    # update necessary pointers in header (0xcc and higher in M-RAW header)
+    foreach $offset (0x5c, 0x64, 0x78, 0x80, 0xcc, 0x114, 0x164) {
         last if $offset >= $jpos;   # some versions have a short header
         my $oldPtr = Get32u(\$hdr, $offset);
         next unless $oldPtr;        # don't update if pointer is zero
-        Set32u($oldPtr + $ptrDiff, \$hdr, $offset);
+        my $newPtr = $oldPtr + $ptrDiff;
+        if ($newPtr < 0 or $newPtr > 0xffffffff) {
+            $offset < 0xcc and $et->Error('Invalid offset in RAF header'), return 1;
+            # assume values at 0xcc and greater are 8-byte integers (NC)
+            # and adjust high word if necessary
+            my $high = Get32u(\$hdr, $offset-4);
+            if ($newPtr < 0) {
+                $high -= 1;
+                $newPtr += 0xffffffff + 1;
+                $high < 0 and $et->Error('RAF header offset error'), return 1;
+            } else {
+                $high += 1;
+                $newPtr -= 0xffffffff + 1;
+            }
+            Set32u($high, \$hdr, $offset-4);
+        }
+        Set32u($newPtr, \$hdr, $offset);
     }
     # write the new header
     my $outfile = $$dirInfo{OutFile};
@@ -1590,35 +1837,41 @@ sub ProcessRAF($$)
     my ($buff, $jpeg, $warn, $offset);
 
     my $raf = $$dirInfo{RAF};
-    $raf->Read($buff,0x5c) == 0x5c    or return 0;
+    $raf->Read($buff,0x70) == 0x70    or return 0;
     $buff =~ /^FUJIFILM/              or return 0;
+    # get position and size of M-RAW header and jpeg preview
+    my ($mpos, $mlen) = unpack('x72NN', $buff);
     my ($jpos, $jlen) = unpack('x84NN', $buff);
     $jpos & 0x8000                   and return 0;
-    $raf->Seek($jpos, 0)              or return 0;
-    $raf->Read($jpeg, $jlen) == $jlen or return 0;
-
+    if ($jpos) {
+        $raf->Seek($jpos, 0)              or return 0;
+        $raf->Read($jpeg, $jlen) == $jlen or return 0;
+    }
+    SetByteOrder('MM');
     $et->SetFileType();
-    $et->FoundTag('RAFVersion', substr($buff, 0x3c, 4));
-
+    my $tbl = GetTagTable('Image::ExifTool::FujiFilm::RAFHeader');
+    $et->ProcessDirectory({ DataPt => \$buff, DirName => 'RAFHeader' }, $tbl);
+    
     # extract information from embedded JPEG
     my %dirInfo = (
         Parent => 'RAF',
-        RAF    => new File::RandomAccess(\$jpeg),
+        RAF    => File::RandomAccess->new(\$jpeg),
     );
-    $$et{BASE} += $jpos;
-    my $rtnVal = $et->ProcessJPEG(\%dirInfo);
-    $$et{BASE} -= $jpos;
-    $et->FoundTag('PreviewImage', \$jpeg) if $rtnVal;
-
+    if ($jpos) {
+        $$et{BASE} += $jpos;
+        my $ok = $et->ProcessJPEG(\%dirInfo);
+        $$et{BASE} -= $jpos;
+        $et->FoundTag('PreviewImage', \$jpeg) if $ok;
+    }
     # extract information from Fuji RAF and TIFF directories
     my ($rafNum, $ifdNum) = ('','');
-    foreach $offset (0x5c, 0x64, 0x78, 0x80) {
-        last if $offset >= $jpos;
-        unless ($raf->Seek($offset, 0) and $raf->Read($buff, 4)) {
+    foreach $offset (0x48, 0x5c, 0x64, 0x78, 0x80) {
+        last if $jpos and $offset >= $jpos;
+        unless ($raf->Seek($offset, 0) and $raf->Read($buff, 8)) {
             $warn = 1;
             last;
         }
-        my $start = unpack('N',$buff);
+        my ($start, $len) = unpack('N2',$buff);
         next unless $start;
         if ($offset == 0x64 or $offset == 0x80) {
             # parse FujiIFD directory
@@ -1629,9 +1882,20 @@ sub ProcessRAF($$)
             $$et{SET_GROUP1} = "FujiIFD$ifdNum";
             my $tagTablePtr = GetTagTable('Image::ExifTool::FujiFilm::IFD');
             # this is TIFF-format data only for some models, so no warning if it fails
-            $et->ProcessTIFF(\%dirInfo, $tagTablePtr, \&Image::ExifTool::ProcessTIFF);
+            unless ($et->ProcessTIFF(\%dirInfo, $tagTablePtr, \&Image::ExifTool::ProcessTIFF)) {
+                # do hash of image data if necessary
+                $et->ImageDataHash($raf, $len, 'raw') if $$et{ImageDataHash} and $raf->Seek($start,0);
+            }
             delete $$et{SET_GROUP1};
             $ifdNum = ($ifdNum || 1) + 1;
+        } elsif ($offset == 0x48) {
+            $$et{VALUE}{FileType} .= ' (M-RAW)';
+            if ($raf->Seek($start, 0) and $raf->Read($buff, $mlen) == $mlen) {
+                my $tbl = GetTagTable('Image::ExifTool::FujiFilm::MRAW');
+                $et->ProcessDirectory({ DataPt => \$buff, DataPos => $start, DirName => 'M-RAW' }, $tbl);
+            } else {
+                $et->Warn('Error reading M-RAW header');
+            }
         } else {
             # parse RAF directory
             %dirInfo = (
@@ -1640,14 +1904,17 @@ sub ProcessRAF($$)
             );
             $$et{SET_GROUP1} = "RAF$rafNum";
             my $tagTablePtr = GetTagTable('Image::ExifTool::FujiFilm::RAF');
-            $et->ProcessDirectory(\%dirInfo, $tagTablePtr) or $warn = 1;
+            if ($et->ProcessDirectory(\%dirInfo, $tagTablePtr)) {
+                $rafNum = ($rafNum || 1) + 1;
+            } else {
+                $warn = 1;
+            }
             delete $$et{SET_GROUP1};
-            $rafNum = ($rafNum || 1) + 1;
         }
     }
     $warn and $et->Warn('Possibly corrupt RAF information');
 
-    return $rtnVal;
+    return 1;
 }
 
 1; # end
@@ -1670,7 +1937,7 @@ FujiFilm maker notes in EXIF information, and to read/write FujiFilm RAW
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
