@@ -22,13 +22,13 @@ use vars qw($VERSION %samsungLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.50';
+$VERSION = '1.58';
 
 sub WriteSTMN($$$);
 sub ProcessINFO($$$);
 sub ProcessSamsungMeta($$$);
 sub ProcessSamsungIFD($$$);
-sub ProcessSamsung($$$);
+sub ProcessSamsung($$;$);
 
 # Samsung LensType lookup
 %samsungLensTypes = (
@@ -454,6 +454,8 @@ my %formatMinMax = (
     0xa018 => { #1
         Name => 'ExposureTime',
         Writable => 'rational64u',
+        ValueConv => '$val=~s/ .*//; $val', # some models write 2 values here
+        ValueConvInv => '$val',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
         PrintConvInv => '$val',
     },
@@ -461,6 +463,8 @@ my %formatMinMax = (
         Name => 'FNumber',
         Priority => 0,
         Writable => 'rational64u',
+        ValueConv => '$val=~s/ .*//; $val', # some models write 2 values here
+        ValueConvInv => '$val',
         PrintConv => 'sprintf("%.1f",$val)',
         PrintConvInv => '$val',
     },
@@ -900,9 +904,10 @@ my %formatMinMax = (
         Name => 'SamsungSvss',
         SubDirectory => { TagTable => 'Image::ExifTool::Samsung::svss' },
     },
+    mdln => 'SamsungModel', #PH (Samsung SM-A136U, etc)
     # swtr - 4 bytes, all zero
     # scid - 8 bytes, all zero
-    # saut - 4 bytes, all zero
+    # saut - 4 or 6 bytes, all zero
 );
 
 # Samsung MP4 svss information (PH - from SM-C101 sample)
@@ -938,25 +943,25 @@ my %formatMinMax = (
 );
 
 # information extracted from Samsung trailer (ie. Samsung SM-T805 "Sound & Shot" JPEG) (ref PH)
+# NOTE: These tags may use $$self{SamsungTagName} in a Condition statement
+#       if necessary to differentiate tags with the same ID but different names
 %Image::ExifTool::Samsung::Trailer = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Other' },
     VARS => { NO_ID => 1, HEX_ID => 0 },
     PROCESS_PROC => \&ProcessSamsung,
+    TAG_PREFIX => 'SamsungTrailer',
     PRIORITY => 0, # (first one takes priority so DepthMapWidth/Height match first DepthMapData)
     NOTES => q{
-        Tags extracted from the trailer of JPEG images written when using certain
-        features (such as "Sound & Shot" or "Shot & More") from Samsung models such
-        as the Galaxy S4 and Tab S, and from the 'sefd' atom in HEIC images from the
-        Samsung S10+.
-    },
-    '0x0001-name' => {
-        Name => 'EmbeddedImageName', # ("DualShot_1","DualShot_2")
-        RawConv => '$$self{EmbeddedImageName} = $val',
+        Tags extracted from the SEFT trailer of JPEG and PNG images written when
+        using certain features (such as "Sound & Shot" or "Shot & More") from
+        Samsung models such as the Galaxy S4 and Tab S, and from the 'sefd' atom in
+        HEIC images from models such as the S10+.
     },
+    '0x0001-name' => 'EmbeddedImageName', # ("DualShot_1","DualShot_2","SingleShot")
     '0x0001' => [
         {
             Name => 'EmbeddedImage',
-            Condition => '$$self{EmbeddedImageName} eq "DualShot_1"',
+            Condition => '$$self{SamsungTagName} ne "DualShot_2"',
             Groups => { 2 => 'Preview' },
             Binary => 1,
         },
@@ -965,6 +970,7 @@ my %formatMinMax = (
             Groups => { 2 => 'Preview' },
             Binary => 1,
         },
+        # (have also seen the string "BOKEH" here (SM-A226B)
     ],
     '0x0100-name' => 'EmbeddedAudioFileName', # ("SoundShot_000")
     '0x0100' => { Name => 'EmbeddedAudioFile', Groups => { 2 => 'Audio' }, Binary => 1 },
@@ -974,6 +980,7 @@ my %formatMinMax = (
    # 0x0800 - unknown (29 bytes) (contains already-extracted EmbeddedAudioFileName)
    # 0x0830-name - seen '1165724808.pre'
    # 0x0830 - unknown (164004 bytes)
+   # 0x08c0-name - seen 'Auto_Enhance_Info' #forum16086
    # 0x08d0-name - seen 'Interactive_Panorama_Info'
    # 0x08d0 - unknown (7984 bytes)
    # 0x08e0-name - seen 'Panorama_Shot_Info'
@@ -983,20 +990,254 @@ my %formatMinMax = (
    # 0x0910 - string, seen 'Front_Cam_Selfie_Info'
    # 0x09e0-name - seen 'Burst_Shot_Info'
    # 0x09e0 - string, seen '489489125'
+   # 0x09e1-name - seen 'BurstShot_Best_Photo_Info' #forum16086
+   # 0x09f0-name - seen 'Pro_Mode_Info' #forum16086
    # 0x0a01-name - seen 'Image_UTC_Data'
     '0x0a01' => { #forum7161
         Name => 'TimeStamp',
         Groups => { 2 => 'Time' },
-        ValueConv => 'ConvertUnixTime($val / 1e3, 1)',
+        ValueConv => 'ConvertUnixTime($val / 1e3, 1, 3)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
     '0x0a20-name' => 'DualCameraImageName', # ("FlipPhoto_002")
     '0x0a20' => { Name => 'DualCameraImage', Groups => { 2 => 'Preview' }, Binary => 1 },
     '0x0a30-name' => 'EmbeddedVideoType', # ("MotionPhoto_Data")
     '0x0a30' => { Name => 'EmbeddedVideoFile', Groups => { 2 => 'Video' }, Binary => 1 }, #forum7161
+   # 0x0a41-name - seen 'BackupRestore_Data' #forum16086
    # 0x0aa1-name - seen 'MCC_Data'
    # 0x0aa1 - seen '204','222','234','302','429'
-    '0x0aa1' => 'MCCData', # (unknown meaning)
+    '0x0aa1' => {
+        Name => 'MCCData',
+        Groups => { 2 => 'Location' },
+        PrintConv => {
+            202 => 'Greece (202)',
+            204 => 'Netherlands (204)',
+            206 => 'Belgium (206)',
+            208 => 'France (208)',
+            212 => 'Monaco (212)',
+            213 => 'Andorra (213)',
+            214 => 'Spain (214)',
+            216 => 'Hungary (216)',
+            218 => 'Bosnia & Herzegov. (218)',
+            219 => 'Croatia (219)',
+            220 => 'Serbia (220)',
+            221 => 'Kosovo (221)',
+            222 => 'Italy (222)',
+            226 => 'Romania (226)',
+            228 => 'Switzerland (228)',
+            230 => 'Czech Rep. (230)',
+            231 => 'Slovakia (231)',
+            232 => 'Austria (232)',
+            234 => 'United Kingdom (234)',
+            235 => 'United Kingdom (235)',
+            238 => 'Denmark (238)',
+            240 => 'Sweden (240)',
+            242 => 'Norway (242)',
+            244 => 'Finland (244)',
+            246 => 'Lithuania (246)',
+            247 => 'Latvia (247)',
+            248 => 'Estonia (248)',
+            250 => 'Russian Federation (250)',
+            255 => 'Ukraine (255)',
+            257 => 'Belarus (257)',
+            259 => 'Moldova (259)',
+            260 => 'Poland (260)',
+            262 => 'Germany (262)',
+            266 => 'Gibraltar (266)',
+            268 => 'Portugal (268)',
+            270 => 'Luxembourg (270)',
+            272 => 'Ireland (272)',
+            274 => 'Iceland (274)',
+            276 => 'Albania (276)',
+            278 => 'Malta (278)',
+            280 => 'Cyprus (280)',
+            282 => 'Georgia (282)',
+            283 => 'Armenia (283)',
+            284 => 'Bulgaria (284)',
+            286 => 'Turkey (286)',
+            288 => 'Faroe Islands (288)',
+            289 => 'Abkhazia (289)',
+            290 => 'Greenland (290)',
+            292 => 'San Marino (292)',
+            293 => 'Slovenia (293)',
+            294 => 'Macedonia (294)',
+            295 => 'Liechtenstein (295)',
+            297 => 'Montenegro (297)',
+            302 => 'Canada (302)',
+            308 => 'St. Pierre & Miquelon (308)',
+            310 => 'United States / Guam (310)',
+            311 => 'United States / Guam (311)',
+            312 => 'United States (312)',
+            316 => 'United States (316)',
+            330 => 'Puerto Rico (330)',
+            334 => 'Mexico (334)',
+            338 => 'Jamaica (338)',
+            340 => 'French Guiana / Guadeloupe / Martinique (340)',
+            342 => 'Barbados (342)',
+            344 => 'Antigua and Barbuda (344)',
+            346 => 'Cayman Islands (346)',
+            348 => 'British Virgin Islands (348)',
+            350 => 'Bermuda (350)',
+            352 => 'Grenada (352)',
+            354 => 'Montserrat (354)',
+            356 => 'Saint Kitts and Nevis (356)',
+            358 => 'Saint Lucia (358)',
+            360 => 'St. Vincent & Gren. (360)',
+            362 => 'Bonaire, Sint Eustatius and Saba / Curacao / Netherlands Antilles (362)',
+            363 => 'Aruba (363)',
+            364 => 'Bahamas (364)',
+            365 => 'Anguilla (365)',
+            366 => 'Dominica (366)',
+            368 => 'Cuba (368)',
+            370 => 'Dominican Republic (370)',
+            372 => 'Haiti (372)',
+            374 => 'Trinidad and Tobago (374)',
+            376 => 'Turks and Caicos Islands / US Virgin Islands (376)',
+            400 => 'Azerbaijan (400)',
+            401 => 'Kazakhstan (401)',
+            402 => 'Bhutan (402)',
+            404 => 'India (404)',
+            405 => 'India (405)',
+            410 => 'Pakistan (410)',
+            412 => 'Afghanistan (412)',
+            413 => 'Sri Lanka (413)',
+            414 => 'Myanmar (Burma) (414)',
+            415 => 'Lebanon (415)',
+            416 => 'Jordan (416)',
+            417 => 'Syrian Arab Republic (417)',
+            418 => 'Iraq (418)',
+            419 => 'Kuwait (419)',
+            420 => 'Saudi Arabia (420)',
+            421 => 'Yemen (421)',
+            422 => 'Oman (422)',
+            424 => 'United Arab Emirates (424)',
+            425 => 'Israel / Palestinian Territory (425)',
+            426 => 'Bahrain (426)',
+            427 => 'Qatar (427)',
+            428 => 'Mongolia (428)',
+            429 => 'Nepal (429)',
+            430 => 'United Arab Emirates (430)',
+            431 => 'United Arab Emirates (431)',
+            432 => 'Iran (432)',
+            434 => 'Uzbekistan (434)',
+            436 => 'Tajikistan (436)',
+            437 => 'Kyrgyzstan (437)',
+            438 => 'Turkmenistan (438)',
+            440 => 'Japan (440)',
+            441 => 'Japan (441)',
+            450 => 'South Korea (450)',
+            452 => 'Viet Nam (452)',
+            454 => 'Hongkong, China (454)',
+            455 => 'Macao, China (455)',
+            456 => 'Cambodia (456)',
+            457 => 'Laos P.D.R. (457)',
+            460 => 'China (460)',
+            466 => 'Taiwan (466)',
+            467 => 'North Korea (467)',
+            470 => 'Bangladesh (470)',
+            472 => 'Maldives (472)',
+            502 => 'Malaysia (502)',
+            505 => 'Australia (505)',
+            510 => 'Indonesia (510)',
+            514 => 'Timor-Leste (514)',
+            515 => 'Philippines (515)',
+            520 => 'Thailand (520)',
+            525 => 'Singapore (525)',
+            528 => 'Brunei Darussalam (528)',
+            530 => 'New Zealand (530)',
+            537 => 'Papua New Guinea (537)',
+            539 => 'Tonga (539)',
+            540 => 'Solomon Islands (540)',
+            541 => 'Vanuatu (541)',
+            542 => 'Fiji (542)',
+            544 => 'American Samoa (544)',
+            545 => 'Kiribati (545)',
+            546 => 'New Caledonia (546)',
+            547 => 'French Polynesia (547)',
+            548 => 'Cook Islands (548)',
+            549 => 'Samoa (549)',
+            550 => 'Micronesia (550)',
+            552 => 'Palau (552)',
+            553 => 'Tuvalu (553)',
+            555 => 'Niue (555)',
+            602 => 'Egypt (602)',
+            603 => 'Algeria (603)',
+            604 => 'Morocco (604)',
+            605 => 'Tunisia (605)',
+            606 => 'Libya (606)',
+            607 => 'Gambia (607)',
+            608 => 'Senegal (608)',
+            609 => 'Mauritania (609)',
+            610 => 'Mali (610)',
+            611 => 'Guinea (611)',
+            612 => 'Ivory Coast (612)',
+            613 => 'Burkina Faso (613)',
+            614 => 'Niger (614)',
+            615 => 'Togo (615)',
+            616 => 'Benin (616)',
+            617 => 'Mauritius (617)',
+            618 => 'Liberia (618)',
+            619 => 'Sierra Leone (619)',
+            620 => 'Ghana (620)',
+            621 => 'Nigeria (621)',
+            622 => 'Chad (622)',
+            623 => 'Central African Rep. (623)',
+            624 => 'Cameroon (624)',
+            625 => 'Cape Verde (625)',
+            626 => 'Sao Tome & Principe (626)',
+            627 => 'Equatorial Guinea (627)',
+            628 => 'Gabon (628)',
+            629 => 'Congo, Republic (629)',
+            630 => 'Congo, Dem. Rep. (630)',
+            631 => 'Angola (631)',
+            632 => 'Guinea-Bissau (632)',
+            633 => 'Seychelles (633)',
+            634 => 'Sudan (634)',
+            635 => 'Rwanda (635)',
+            636 => 'Ethiopia (636)',
+            637 => 'Somalia (637)',
+            638 => 'Djibouti (638)',
+            639 => 'Kenya (639)',
+            640 => 'Tanzania (640)',
+            641 => 'Uganda (641)',
+            642 => 'Burundi (642)',
+            643 => 'Mozambique (643)',
+            645 => 'Zambia (645)',
+            646 => 'Madagascar (646)',
+            647 => 'Reunion (647)',
+            648 => 'Zimbabwe (648)',
+            649 => 'Namibia (649)',
+            650 => 'Malawi (650)',
+            651 => 'Lesotho (651)',
+            652 => 'Botswana (652)',
+            653 => 'Swaziland (653)',
+            654 => 'Comoros (654)',
+            655 => 'South Africa (655)',
+            657 => 'Eritrea (657)',
+            659 => 'South Sudan (659)',
+            702 => 'Belize (702)',
+            704 => 'Guatemala (704)',
+            706 => 'El Salvador (706)',
+            708 => 'Honduras (708)',
+            710 => 'Nicaragua (710)',
+            712 => 'Costa Rica (712)',
+            714 => 'Panama (714)',
+            716 => 'Peru (716)',
+            722 => 'Argentina Republic (722)',
+            724 => 'Brazil (724)',
+            730 => 'Chile (730)',
+            732 => 'Colombia (732)',
+            734 => 'Venezuela (734)',
+            736 => 'Bolivia (736)',
+            738 => 'Guyana (738)',
+            740 => 'Ecuador (740)',
+            744 => 'Paraguay (744)',
+            746 => 'Suriname (746)',
+            748 => 'Uruguay (748)',
+            750 => 'Falkland Islands (Malvinas) (750)',
+            901 => 'International Networks / Satellite Networks (901)',
+        },
+    },
    # 0x0ab0-name - seen 'DualShot_Meta_Info'
     '0x0ab1-name' => {
         Name => 'DepthMapName',
@@ -1018,16 +1259,49 @@ my %formatMinMax = (
         Name => 'DualShotExtra',
         SubDirectory => { TagTable => 'Image::ExifTool::Samsung::DualShotExtra' },
      },
+   # 0x0ab4-name - seen 'DualShot_Core_Info' #forum16086
    # 0x0ac0-name - seen 'ZoomInOut_Info' (SM-N950U)
    # 0x0ac0 - 2048 bytes of interesting stuff including firmware version? (SM-N950U)
+   # 0x0b30-name - seen 'Camera_Sticker_Info' #forum16086
     '0x0b40' => { # (SM-N975X front camera)
         Name => 'SingleShotMeta',
         SubDirectory => { TagTable => 'Image::ExifTool::Samsung::SingleShotMeta' },
+        # (have also see the string "BOKEH_INFO" here (SM-A226B)
      },
    # 0x0b41-name - seen 'SingeShot_DepthMap_1' (Yes, "Singe") (SM-N975X front camera)
     '0x0b41' => { Name => 'SingleShotDepthMap', Binary => 1 },
+   # 0x0b51-name - seen 'Intelligent_PhotoEditor_Data' #forum16086
+   # 0x0b60-name - seen 'UltraWide_PhotoEditor_Data' #forum16086
+   # 0x0b90-name - seen 'Document_Scan_Info' #forum16086
+   # 0x0ba1-name - seen 'Original_Path_Hash_Key', 'PhotoEditor_Re_Edit_Data', 'deco_doodle_bitmap', 'deco_sticker_bitmap', 'deco_text_bitmap'
+   # 0x0ba2-name - seen 'Copy_Available_Edit_Info' #forum16086
+   # 0x0bc0-name - seen 'Single_Relighting_Bokeh_Info' #forum16086
+   # 0x0bd0-name - seen 'Dual_Relighting_Bokeh_Info' #forum16086
+   # 0x0be0-name - seen 'Livefocus_JDM_Info' #forum16086
+   # 0x0bf0-name - seen 'Remaster_Info' #forum16086
+   '0x0bf0' => 'RemasterInfo', #forum16086/16242
+   # 0x0c21-name - seen 'Portrait_Effect_Info' #forum16086
+   # 0x0c51-name - seen 'Samsung_Capture_Info' #forum16086
+   '0x0c51' => 'SamsungCaptureInfo', #forum16086/16242
+   # 0x0c61-name - seen 'Camera_Capture_Mode_Info' #forum16086
+   # 0x0c71-name - seen 'Pro_White_Balance_Info' #forum16086
+   # 0x0c81-name - seen 'Watermark_Info' #forum16086
+   # 0x0cc1-name - seen 'Color_Display_P3' #forum16086
+   # 0x0cd2-name - seen 'Photo_HDR_Info' #forum16086
+   # 0x0ce1-name - seen 'Gallery_DC_Data' #forum16086
+   # 0x0d01-name - seen 'Camera_Scene_Info', 'Camera_Scene_Info2', 'Camera_Scene_Info3' #forum16086
+   # 0x0d11-name - seen 'Video_Snapshot_Info' #forum16086
+   # 0x0d21-name - seen 'Camera_Scene_Info' #forum16086
+   # 0x0d31-name - seen 'Food_Blur_Effect_Info' #forum16086
+    '0x0d91' => { #forum16086/16242
+        Name => 'PEg_Info',
+        Description => 'PEg Info',
+        SubDirectory => { TagTable => 'Image::ExifTool::JSON::Main' },
+    },
+   # 0x0da1-name - seen 'Captured_App_Info' #forum16086
    # 0xa050-name - seen 'Jpeg360_2D_Info' (Samsung Gear 360)
    # 0xa050 - seen 'Jpeg3602D' (Samsung Gear 360)
+   # 0x0c81-name - seen 'Watermark_Info'
 );
 
 # DualShot Extra Info (ref PH)
@@ -1221,7 +1495,7 @@ sub ProcessSamsungMeta($$$)
     my $pos = $$dirInfo{DirStart};
     my $end = $$dirInfo{DirLen} + $pos;
     unless ($pos + 8 <= $end and substr($$dataPt, $pos, 4) eq 'DOFS') {
-        $et->Warn("Unrecognized $dirName data");
+        $et->Warn("Unrecognized $dirName data", 1);
         return 0;
     }
     my $ver = Get32u($dataPt, $pos + 4);
@@ -1297,7 +1571,7 @@ sub ProcessSamsungIFD($$$)
 # Returns: 1 on success, 0 not valid Samsung trailer, or -1 error writing
 # - updates DataPos to point to start of Samsung trailer
 # - updates DirLen to existing trailer length
-sub ProcessSamsung($$$)
+sub ProcessSamsung($$;$)
 {
     my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
@@ -1308,7 +1582,7 @@ sub ProcessSamsung($$$)
     my ($buff, $buf2, $index, $offsetPos, $audioNOff, $audioSize);
 
     unless ($raf) {
-        $raf = new File::RandomAccess($$dirInfo{DataPt});
+        $raf = File::RandomAccess->new($$dirInfo{DataPt});
         $et->VerboseDir('SamsungTrailer');
     }
     return 0 unless $raf->Seek(-6-$offset, 2) and $raf->Read($buff, 6) == 6 and
@@ -1387,8 +1661,13 @@ SamBlock:
                 $audioSize = $size - 8 - $len;
                 next;
             }
-            # add unknown tags if necessary
+            last unless $raf->Seek($dirPos-$noff, 0) and $raf->Read($buf2, $size) == $size;
+            # (could validate the first 4 bytes of the block because they
+            # are the same as the first 4 bytes of the directory entry)
+            $len = Get32u(\$buf2, 4);
+            last if $len + 8 > $size;
             my $tag = sprintf("0x%.4x", $type);
+            # add unknown tags if necessary
             unless ($$tagTablePtr{$tag}) {
                 next unless $unknown or $verbose;
                 my %tagInfo = (
@@ -1407,11 +1686,8 @@ SamBlock:
                 );
                 AddTagToTable($tagTablePtr, "$tag-name", \%tagInfo2);
             }
-            last unless $raf->Seek($dirPos-$noff, 0) and $raf->Read($buf2, $size) == $size;
-            # (could validate the first 4 bytes of the block because they
-            # are the same as the first 4 bytes of the directory entry)
-            $len = Get32u(\$buf2, 4);
-            last if $len + 8 > $size;
+            # set SamsungTagName ExifTool member for use in tag Condition
+            $$et{SamsungTagName} = substr($buf2, 8, $len);
             # extract tag name and value
             $et->HandleTag($tagTablePtr, "$tag-name", undef,
                 DataPt  => \$buf2,
@@ -1425,6 +1701,7 @@ SamBlock:
                 Start   => 8 + $len,
                 Size    => $size - (8 + $len),
             );
+            delete $$et{SamsungTagName};
         }
         if ($outfile) {
             last unless $raf->Seek($dataPos, 0) and $raf->Read($buff, $dirLen) == $dirLen;
@@ -1437,7 +1714,7 @@ SamBlock:
                 # add a fixup so the calling routine can apply further shifts if necessary
                 require Image::ExifTool::Fixup;
                 my $fixup = $$dirInfo{Fixup};
-                $fixup or $fixup = $$dirInfo{Fixup} = new Image::ExifTool::Fixup;
+                $fixup or $fixup = $$dirInfo{Fixup} = Image::ExifTool::Fixup->new;
                 $fixup->AddFixup(length($buff) - $offsetPos);
                 $fixup->AddFixup(length($buff) - $offsetPos + 4);
             }
@@ -1459,7 +1736,7 @@ sub WriteSTMN($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
     # create a Fixup for the PreviewImage
-    $$dirInfo{Fixup} = new Image::ExifTool::Fixup;
+    $$dirInfo{Fixup} = Image::ExifTool::Fixup->new;
     my $val = Image::ExifTool::WriteBinaryData($et, $dirInfo, $tagTablePtr);
     # force PreviewImage into the trailer even if it fits in EXIF segment
     $$et{PREVIEW_INFO}{IsTrailer} = 1 if $$et{PREVIEW_INFO};
@@ -1485,7 +1762,7 @@ Samsung maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
