@@ -148,6 +148,7 @@ sub PrintInvGPSCoordinates($)
         $v[2] = Image::ExifTool::ToFloat($v[2]) * ($below ? -1 : 1) if @v == 3;
         return "@v";
     }
+    return $val if $val =~ /^([-+]?\d+(\.\d*)?)\s+([-+]?\d+(\.\d*)?)$/; # already 2 floats?
     return $val if $val =~ /^([-+]\d+(\.\d*)?){2,3}(CRS.*)?\/?$/; # already in ISO6709 format?
     return undef;
 }
@@ -167,7 +168,8 @@ sub ConvInvISO6709($)
         # requires at least 3 digits after the decimal point
         # (and as of Apr 2021, Google Photos doesn't accept coordinats
         #  with more than 5 digits after the decimal place:
-        #  https://exiftool.org/forum/index.php?topic=11055.msg67171#msg67171 )
+        #  https://exiftool.org/forum/index.php?topic=11055.msg67171#msg67171 
+        #  still a problem Apr 2024: https://exiftool.org/forum/index.php?msg=85761)
         my @fmt = ('%s%02d.%s%s','%s%03d.%s%s','%s%d.%s%s');
         my @limit = (90,180);
         foreach (@a) {
@@ -505,10 +507,16 @@ sub WriteItemInfo($$$)
         my ($warn, $extent, $buff, @edit);
         $warn = 'Missing iloc box' unless $$boxPos{iloc};
         $warn = "No Extents for $type item" unless $$item{Extents} and @{$$item{Extents}};
-        $warn = "Can't currently decode encoded $type metadata" if $$item{ContentEncoding};
+        if ($$item{ContentEncoding}) {
+            if ($$item{ContentEncoding} ne 'deflate') {
+                $warn = "Can't currently decode $$item{ContentEncoding} encoded $type metadata";
+            } elsif (not eval { require Compress::Zlib }) {
+                $warn = "Install Compress::Zlib to decode deflated $type metadata";
+            }
+        }
         $warn = "Can't currently decode protected $type metadata" if $$item{ProtectionIndex};
         $warn = "Can't currently extract $type with construction method $$item{ConstructionMethod}" if $$item{ConstructionMethod};
-        $warn = "$type metadata is not this file" if $$item{DataReferenceIndex};
+        $warn = "$type metadata is not in this file" if $$item{DataReferenceIndex};
         $warn and $et->Warn($warn), next;
         my $base = $$item{BaseOffset} || 0;
         my $val = '';
@@ -525,6 +533,25 @@ sub WriteItemInfo($$$)
         }
         next unless defined $buff;
         $buff = $val . $buff if length $val;
+        my $comp = $et->Options('Compress');
+        if (defined $comp and ($comp xor $$item{ContentEncoding})) {
+            #TODO: add ability to edit infe entry in iinf to change encoding according to Compress option if set
+            $et->Warn("Can't currently change compression when rewriting $name in HEIC",1);
+        }
+        my $wasDeflated;
+        if ($$item{ContentEncoding}) {
+            my ($v2, $stat);
+            my $inflate = Compress::Zlib::inflateInit();
+            $inflate and ($v2, $stat) = $inflate->inflate($buff);
+            $et->VPrint(0, "  (Inflating stored $name metadata)\n");
+            if ($inflate and $stat == Compress::Zlib::Z_STREAM_END()) {
+                $buff = $v2;
+                $wasDeflated = 1;
+            } else {
+                $et->WarnOnce("Error inflating $name metadata");
+                next;
+            }
+        }
         my ($hdr, $subTable, $proc);
         if ($name eq 'EXIF') {
             if (not length $buff) {
@@ -558,6 +585,17 @@ sub WriteItemInfo($$$)
             ($dirInfo{DirLen} or length $newVal))
         {
             $newVal = $hdr . $newVal if length $hdr and length $newVal;
+            if ($wasDeflated) {
+                my $deflate = Compress::Zlib::deflateInit();
+                if ($deflate) {
+                    $et->VPrint(0, "  (Re-deflating new $name metadata)\n");
+                    $buff = $deflate->deflate($newVal);
+                    if (defined $buff) {
+                        $buff .= $deflate->flush();
+                        $newVal = $buff;
+                    }
+                }
+            }
             $edit[0][2] = \$newVal;     # replace the old chunk with the new data
             $edit[0][3] = $id;          # mark this chunk with the item ID
             push @mdatEdit, @edit;
@@ -638,21 +676,38 @@ sub WriteItemInfo($$$)
             # add new infe to iinf
             $add{iinf} = $add{iref} = $add{iloc} = '' unless defined $add{iinf};
             my ($type, $mime);
+            my $enc = '';
             if ($name eq 'XMP') {
                 $type = "mime\0";
                 $mime = "application/rdf+xml\0";
+                # write compressed XMP if Compress option is set
+                if ($et->Options('Compress') and length $newVal) {
+                    if (not eval { require Compress::Zlib }) {
+                        $et->WarnOnce('Install Compress::Zlib to write compressed metadata');
+                    } else {
+                        my $deflate = Compress::Zlib::deflateInit();
+                        if ($deflate) {
+                            $et->VPrint(0, "  (Deflating new $name metadata)\n");
+                            my $buff = $deflate->deflate($newVal);
+                            if (defined $buff) {
+                                $newVal = $buff . $deflate->flush();
+                                $enc = "deflate\0";
+                            }
+                        }
+                    }
+                }
             } else {
                 $type = "Exif\0";
                 $mime = '';
             }
             my $id = 1;
             ++$id while $$items{$id} or $usedID{$id};   # find next unused item ID
-            my $n = length($type) + length($mime) + 16;
+            my $n = length($type) + length($mime) + length($enc) + 16;
             if ($id < 0x10000) {
-                $add{iinf} .= pack('Na4CCCCnn', $n, 'infe', 2, 0, 0, 1, $id, 0) . $type . $mime;
+                $add{iinf} .= pack('Na4CCCCnn', $n, 'infe', 2, 0, 0, 1, $id, 0) . $type . $mime . $enc;
             } else {
                 $n += 2;
-                $add{iinf} .= pack('Na4CCCCNn', $n, 'infe', 3, 0, 0, 1, $id, 0) . $type . $mime;
+                $add{iinf} .= pack('Na4CCCCNn', $n, 'infe', 3, 0, 0, 1, $id, 0) . $type . $mime . $enc;
             }
             # add new cdsc to iref
             if ($irefVer) {
@@ -731,28 +786,36 @@ sub WriteItemInfo($$$)
                 $add{$tag} = Set32u(12 + length $add{$tag}) . $tag .
                              Set8u($$boxPos{$tag}[2]) . "\0\0\0" . $add{$tag};
             } elsif ($tag ne 'hdlr') {
-                my $n = Get32u($outfile, $pos);
-                Set32u($n + length($add{$tag}), $outfile, $pos);    # increase box size
+                my $n = Get32u($outfile, $pos) +  length($add{$tag});
+                Set32u($n, $outfile, $pos);    # increase box size
             }
             if ($tag eq 'iinf') {
                 my $iinfVer = Get8u($outfile, $pos + 8);
                 if ($iinfVer == 0) {
-                    my $n = Get16u($outfile, $pos + 12);
-                    Set16u($n + $countNew, $outfile, $pos + 12);    # incr count
+                    my $n = Get16u($outfile, $pos + 12) + $countNew;
+                    if ($n > 0xffff) {
+                        $et->Error("Can't currently handle rollover to long item count");
+                        return undef;
+                    }
+                    Set16u($n, $outfile, $pos + 12);    # incr count
                 } else {
-                    my $n = Get32u($outfile, $pos + 12);
-                    Set32u($n + $countNew, $outfile, $pos + 12);    # incr count
+                    my $n = Get32u($outfile, $pos + 12) + $countNew;
+                    Set32u($n, $outfile, $pos + 12);    # incr count
                 }
             } elsif ($tag eq 'iref') {
                 # nothing more to do
             } elsif ($tag eq 'iloc') {
                 my $ilocVer = Get8u($outfile, $pos + 8);
                 if ($ilocVer < 2) {
-                    my $n = Get16u($outfile, $pos + 14);
-                    Set16u($n + $countNew, $outfile, $pos + 14);    # incr count
+                    my $n = Get16u($outfile, $pos + 14) + $countNew;
+                    Set16u($n, $outfile, $pos + 14);    # incr count
+                    if ($n > 0xffff) {
+                        $et->Error("Can't currently handle rollover to long item count");
+                        return undef;
+                    }
                 } else {
-                    my $n = Get32u($outfile, $pos + 14);
-                    Set32u($n + $countNew, $outfile, $pos + 14);    # incr count
+                    my $n = Get32u($outfile, $pos + 14) + $countNew;
+                    Set32u($n, $outfile, $pos + 14);    # incr count
                 }
                 # must also update pointer locations in this box
                 if ($added) {
@@ -784,6 +847,7 @@ sub WriteQuickTime($$$)
     $et or return 1;    # allow dummy access to autoload this package
     my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $delCount);
     my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err, $atomCount);
+    my ($tag, $lastTag, $errStr);
     my $outfile = $$dirInfo{OutFile} || return 0;
     my $raf = $$dirInfo{RAF};       # (will be null for lower-level atoms)
     my $dataPt = $$dirInfo{DataPt}; # (will be null for top-level atoms)
@@ -856,7 +920,10 @@ sub WriteQuickTime($$$)
     }
     $atomCount = $$tagTablePtr{VARS}{ATOM_COUNT} if $$tagTablePtr{VARS};
 
+    $tag = $lastTag = '';
+
     for (;;) {      # loop through all atoms at this level
+        $lastTag = $tag if $$tagTablePtr{$tag};    # keep track of last known tag
         if (defined $atomCount and --$atomCount < 0 and $dataPt) {
             # stop processing now and just copy the rest of the atom
             Write($outfile, substr($$dataPt, $raf->Tell())) or $rtnVal=$rtnErr, $err=1;
@@ -870,28 +937,31 @@ sub WriteQuickTime($$$)
                 # --> hold this terminator to the end
                 $term = $hdr;
             } elsif ($n != 0) {
-                $et->Error("Unknown $n bytes at end of file", 1);
+                # warn unless this is 1-3 pad bytes
+                $et->Error("Unknown $n bytes at end of file", 1) if $n > 3 or $hdr ne "\0" x $n;
             }
             last;
         }
         my $size = Get32u(\$hdr, 0) - 8;    # (atom size without 8-byte header)
-        my $tag = substr($hdr, 4, 4);
+        $tag = substr($hdr, 4, 4);
         if ($size == -7) {
             # read the extended size
-            $raf->Read($buff, 8) == 8 or $et->Error('Truncated extended atom'), last;
+            $raf->Read($buff, 8) == 8 or $errStr = 'Truncated extended atom', last;
             $hdr .= $buff;
             my ($hi, $lo) = unpack('NN', $buff);
             if ($hi or $lo > 0x7fffffff) {
                 if ($hi > 0x7fffffff) {
-                    $et->Error('Invalid atom size');
+                    $errStr = 'Invalid atom size';
                     last;
                 } elsif (not $et->Options('LargeFileSupport')) {
                     $et->Error('End of processing at large atom (LargeFileSupport not enabled)');
                     last;
+                } elsif ($et->Options('LargeFileSupport') eq '2') {
+                    $et->WarnOnce('Processing large atom (LargeFileSupport is 2)');
                 }
             }
             $size = $hi * 4294967296 + $lo - 16;
-            $size < 0 and $et->Error('Invalid extended atom size'), last;
+            $size < 0 and $errStr = 'Invalid extended atom size', last;
         } elsif ($size == -8) {
             if ($dataPt) {
                 last if $$dirInfo{DirName} eq 'CanonCNTH';  # (this is normal for Canon CNTH atom)
@@ -907,7 +977,7 @@ sub WriteQuickTime($$$)
             }
             last;
         } elsif ($size < 0) {
-            $et->Error('Invalid atom size');
+            $errStr = 'Invalid atom size';
             last;
         }
 
@@ -951,11 +1021,11 @@ sub WriteQuickTime($$$)
                 $tag = PrintableTagID($tag,3);
                 if ($size > $maxReadLen and $got == 0x10000) {
                     my $mb = int($size / 0x100000 + 0.5);
-                    $et->Error("'${tag}' atom is too large for rewriting ($mb MB)");
+                    $errStr = "'${tag}' atom is too large for rewriting ($mb MB)";
                 } else {
-                    $et->Error("Truncated '${tag}' atom");
+                    $errStr = "Truncated '${tag}' atom";
                 }
-                return $rtnVal;
+                last;
             }
         }
         # save the handler type for this track
@@ -1440,11 +1510,35 @@ sub WriteQuickTime($$$)
             $writeLast = ($writeLast || '') . $hdr . $buff;
         } else {
             # save position of this box in the output buffer
+#TODO do this:
+#TODO            my $bp = $boxPos{$tag} || ($boxPos{$tag} = [ ]);
+#TODO            push @$bp, length($$outfile), length($hdr) + length($buff);
+#TODO instead of this:
             $boxPos{$tag} = [ length($$outfile), length($hdr) + length($buff) ];
+#TODO then we have the positions of all the infe boxes -- we then only need
+#TODO to know the index of the box to edit if the encoding changes for one of them
             # copy the existing atom
             Write($outfile, $hdr, $buff) or $rtnVal=$rtnErr, $err=1, last;
         }
     }
+    # ($errStr is set if there was an error that could possibly be due to an unknown trailer)
+    if ($errStr) {
+        if (($lastTag eq 'mdat' or $lastTag eq 'moov') and not $dataPt and (not $$tagTablePtr{$tag} or
+            ref $$tagTablePtr{$tag} eq 'HASH' and $$tagTablePtr{$tag}{Unknown}))
+        {
+            my $nvTrail = $et->GetNewValueHash($Image::ExifTool::Extra{Trailer});
+            if ($$et{DEL_GROUP}{Trailer} or ($nvTrail and not ($$nvTrail{Value} and $$nvTrail{Value}[0]))) {
+                $errStr =~ s/ is too large.*//;
+                $et->Warn('Deleted unknown trailer with ' . lcfirst($errStr));
+            } else {
+                $et->Warn('Unknown trailer with ' . lcfirst($errStr));
+                $et->Error('Use "-trailer=" to delete unknown trailer');
+            }
+        } else {
+            $et->Error($errStr);
+            return $dataPt ? undef : 1;
+        }
+    }
     $et->VPrint(0, "  [deleting $delCount $dirName tag".($delCount==1 ? '' : 's')."]\n") if $delCount;
 
     $createKeys &= ~0x01 unless $$addDirs{Keys};   # (Keys may have been written)
